#!/bin/bash
#
# This script makes preliminary checks for ensuring database access
#
# Author: Abilis <support@antek.it>
# Date:   06/05/2013
# Ver:    3.0.0-1
#
# Changelog
# 3.0.0: First release
#

# --- CONSTANTS DEFINITIONS ---

SCRIPT_FULL_PATH=`readlink -f $0`
SCRIPT_DIR_PATH=`dirname ${SCRIPT_FULL_PATH}`

source ${SCRIPT_DIR_PATH}/include/const.sh

PRIVILEGES="ALTER\nINSERT\nSELECT\nCREATE\nDROP\nLOCK TABLES\nUPDATE"

# --- END CONSTANTS DEFINITIONS ---

# Check if CONFFILE exists
if [[ ! -e ${CONFFILE} ]]; then
  echo -e "\nERROR: File ${CONFFILE} not found\n"
  echo "If collector was installed into a different place please specify the"
  echo "collector.conf path into the CONFFILE variable inside include/const.sh script."
  echo
  $(exit 1)
fi

# Obtain database info from the CONFFILE
MYSQLHOST=`grep MYSQLHOST ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPORT=`grep MYSQLPORT ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLDB=`grep MYSQLDB ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLUSER=`grep MYSQLUSER ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLPASS=`grep MYSQLPASS ${CONFFILE} | awk -F ':' '{print $2}'`
MYSQLTABLES=`egrep ^MYSQLTABLE ${CONFFILE} | sort | uniq | awk -F ':' '{print $2}'`

# Check if required database fields are present
for keyword in MYSQLHOST MYSQLDB MYSQLUSER; do
  if [[ -z ${!keyword} ]]; then
    echo -e "\nERROR: ${keyword} is not specified in ${CONFFILE}!"
    echo -e "Please set the correct db credentials in ${CONFFILE}\n"
    $(exit 2)
  fi
done

# If user is root skip privileges check
if [[ ${MYSQLUSER} -eq "root" ]]; then
  echo -e "\nUsing user \"root\", skipping privileges check.\n"
  $(exit 0)
else 

# Set default database port if not set into collector.conf
if [[ -z ${MYSQLPORT} ]]; then
  echo "INFO: MYSQLPORT is not set, assuming 3306"
  MYSQLPORT=3306
fi

# Assume no password if MYSQLPASS is not set
if [[ -z ${MYSQLPASS} ]]; then
  echo "INFO: MYSQLPASS is not set"
else
  MYSQLPASS="-p${MYSQLPASS}"
fi

MYSQL_CMD="mysql ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS}"

echo -e "\nChecking for privileges of user ${MYSQLUSER} on database ${MYSQLDB}...\n"

GRANTS=$(echo "SHOW GRANTS FOR '${MYSQLUSER}'" | \
        mysql -u${MYSQLUSER} ${MYSQLPASS} | \
        egrep -e "ON (\`${MYSQLDB}\`|\*)\.\*")

if [[ $? -eq "0" ]]; then
  # First check for ALL PRIVILEGES
  echo -e ${GRANTS} | grep "ALL PRIVILEGES" > /dev/null

  if [[ $? -ne "0" ]]; then
    # ALL PRIVILEGES not found. Check for each privilege
    echo -e ${PRIVILEGES} | while read p; do
      echo -e ${GRANTS} | grep "${p}" > /dev/null
      if [[ $? -ne "0" ]]; then
        echo -e "ERROR: user ${MYSQLUSER} doesn't have ${p} privilege on ${MYSQLDB}.\n"
        echo -e "Please ensure ${MYSQLUSER} has all the following privileges"
        echo -e "on database ${MYSQLDB} before install/upgrade:\n"

        echo -e ${PRIVILEGES} | while read p; do
          echo -e "  ${p}"
        done
        echo -e "\nHINT: To grant privileges on database use the following query:\n"
        echo -e "GRANT ALTER,INSERT,SELECT,CREATE,DROP,LOCK TABLES,UPDATE"
        echo -e "  ON  ${MYSQLDB}.* "
        echo -e "  TO '${MYSQLUSER}'@'${MYSQLHOST}' "
        echo -e "  IDENTIFIED BY '<YOUR_USER_PASSWORD>'\n"
        exit 1
      fi
      done

      if [[ $? -eq "1" ]]; then
        exit 1
      fi
  fi

  echo -e "User ${MYSQLUSER} has all required privileges on ${MYSQLDB}. Good!\n";
  $(exit 0)

else 
  echo -e "ERROR: Unable to get grants for user ${MYSQLUSER} on DB ${MYSQLDB}"
  echo -e "Please check error before continue!\n"
  exit 1
fi
fi
